@@ -15,19 +15,6 @@
 #define CMD_KILL		10
 #define CMD_RESUME		11
 
-#define GET_DUMP_FILE(req_cmd)							\
-do {										\
-	dumpfile = param->dumpfile;						\
-	if (dumpfile == NULL) {							\
-		if (cmd == req_cmd) {						\
-			logger(-1,  0, "Error: dumpfile is not specified");	\
-			goto err;						\
-		}								\
-		get_dump_file(veid, param->dumpdir, buf, sizeof(buf));		\
-		dumpfile = buf;							\
-	}									\
-} while (0)
-
 int cpt_cmd(vps_handler *h, envid_t veid, const char *root,
 		int action, int cmd, unsigned int ctx);
 int vps_chkpnt(vps_handler *h, envid_t veid, const fs_param *fs,
