@@ -27,6 +27,7 @@
 #include "types.h"
 #include "quota.h"
 #include "fs.h"
+#include "res.h"
 
 #define VZCTL_VE_ROOTHDD_DIR	"root.hdd"
 
@@ -44,6 +45,7 @@
 struct vzctl_create_image_param {
 	int mode;
 	unsigned long size;
+	unsigned long inodes;	/* number of inodes */
 };
 
 struct vzctl_mount_param {
@@ -60,7 +62,8 @@ struct vzctl_mount_param {
 
 int is_ploop_supported(void);
 int get_ploop_type(const char *type);
-int ve_private_is_ploop(const char *private);
+int guess_ve_private_is_ploop(const char *private);
+int ve_private_is_ploop(const fs_param *fs);
 
 #ifdef HAVE_PLOOP
 extern struct ploop_functions ploop;
@@ -78,7 +81,7 @@ int vzctl_create_image(const char *ve_private, struct vzctl_create_image_param *
 int vzctl_mount_image(const char *ve_private, struct vzctl_mount_param *param);
 int vzctl_umount_image(const char *ve_private);
 int vzctl_convert_image(const char *ve_private, int mode);
-int vzctl_resize_image(const char *ve_private, unsigned long long newsize);
+int vzctl_resize_image(const char *ve_private, unsigned long long newsize, int offline);
 int vzctl_get_ploop_dev(const char *mnt, char *out, int len);
 int vzctl_create_snapshot(const char *ve_private, const char *guid);
 int vzctl_merge_snapshot(const char *ve_private, const char *guid);
@@ -87,7 +90,7 @@ int vzctl_mount_snapshot(unsigned envid, const char *ve_private, struct vzctl_mo
 int vzctl_umount_snapshot(unsigned envid, const char *ve_private, char *guid);
 const char *generate_snapshot_component_name(unsigned int envid,
 		const char *data, char *buf, int len);
-int vzctl_env_convert_ploop(vps_handler *h, envid_t veid,
+int vzctl_env_convert_ploop(vps_handler *h, envid_t veid, vps_param *vps_p,
 		fs_param *fs, dq_param *dq, int mode);
 #endif /* HAVE_PLOOP */
 
