@@ -18,14 +18,14 @@
 #ifndef	_IPTABLES_H_
 #define _IPTABLES_H_
 
-struct iptables_s {
-	char *name;
-	unsigned long id;
-	unsigned long long mask;
-};
+#include "res.h"
 
-struct iptables_s *find_ipt(const char *name);
-void ipt_mask2str(unsigned long mask, char *buf, int size);
-unsigned long long get_ipt_mask(unsigned long mask);
+void ipt_mask2str(unsigned long long mask, char *buf, int size);
+unsigned long long get_ipt_mask(env_param_t *p);
+int parse_iptables(env_param_t *env, char *val);
+
+/* New NETFILTER functionality, obsoleting IPTABLES mask */
+const char* netfilter_mask2str(unsigned long id);
+int parse_netfilter(env_param_t *env, const char *val);
 
 #endif //_IPTABLES_H_
