@@ -126,13 +126,12 @@ typedef struct {
 	char *config;
 	char *origin_sample;
 	char *lockdir;
-	int layout; /* VE_LAYOUT_* */
-	int mode; /* PLOOP_*_MODE */
 	char *apply_cfg;
 	int apply_cfg_map;
 	int reset_ub;
 	int skip_umount;
 	int skip_fsck;
+	int skip_remount;
 } vps_opt;
 
 struct log_s {
@@ -149,6 +148,7 @@ struct vzctl_snapshot_param {
 #define SNAPSHOT_SKIP_SUSPEND	1 << 1
 #define SNAPSHOT_SKIP_CONFIG	1 << 2
 #define SNAPSHOT_SKIP_RESUME	1 << 3
+#define SNAPSHOT_MUST_RESUME	1 << 4
 	int flags;
 	char *target;	/* snapshot mount point */
 };
