@@ -93,13 +93,14 @@ static int download_template(char *tmpl)
 struct destroy_ve {
 	envid_t veid;
 	char *private;
+	int layout;
 };
 
 static void cleanup_destroy_ve(void *data)
 {
 	struct destroy_ve *d = data;
 
-	vps_destroy_dir(d->veid, d->private);
+	vps_destroy_dir(d->veid, d->private, d->layout);
 }
 
 #ifdef HAVE_PLOOP
@@ -109,6 +110,42 @@ static void cleanup_umount_ploop(void *data)
 }
 #endif
 
+/* Check if a directory is a mount point
+ *
+ * Returns:
+ *   1: a mount point
+ *   0: not a mount point
+ *  -1: error
+ */
+int is_mount_point(const char *dir)
+{
+	struct stat st1, st2;
+	char parent[PATH_MAX];
+
+	if (!dir)
+		return -1;
+
+	if (stat(dir, &st1)) {
+		if (errno == ENOENT) /* not a mount point */
+			return 0;
+		logger(-1, errno, "stat(%s)", dir);
+		return -1;
+	}
+
+	if (!S_ISDIR(st1.st_mode)) {
+		logger(-1, 0, "Path %s is not a directory", dir);
+		return -1;
+	}
+
+	snprintf(parent, sizeof(parent), "%s/..", dir);
+	if (stat(parent, &st2)) {
+		logger(-1, errno, "stat(%s)", parent);
+		return -1;
+	}
+
+	return (st1.st_dev != st2.st_dev);
+}
+
 static int fs_create(envid_t veid, vps_handler *h, vps_param *vps_p)
 {
 	char tarball[PATH_LEN];
@@ -119,16 +156,15 @@ static int fs_create(envid_t veid, vps_handler *h, vps_param *vps_p)
 	char *env[6];
 	int quota = 0;
 	int i;
-	char *dst;
+	char *untar_to;
 	const char *ext[] = { "", ".gz", ".bz2", ".xz", NULL };
 	const char *errmsg_ext = "[.gz|.bz2|.xz]";
 	dq_param *dq = &vps_p->res.dq;
-	int layout = vps_p->opt.layout;
 	fs_param *fs = &vps_p->res.fs;
 	tmpl_param *tmpl = &vps_p->res.tmpl;
 	unsigned long uid_offset = 0;
 	unsigned long gid_offset = 0;
-	int ploop = (layout == VE_LAYOUT_PLOOP);
+	int ploop = (fs->layout == VE_LAYOUT_PLOOP);
 	struct destroy_ve ddata;
 	struct vzctl_cleanup_handler *ch;
 #ifdef HAVE_PLOOP
@@ -173,8 +209,18 @@ find:
 				fs->tmpl, tmpl->ostmpl, errmsg_ext);
 		return VZ_OSTEMPLATE_NOT_FOUND;
 	}
+
+	/* Check if VE_PRIVATE is not a mount point (#3166) */
+	ret = is_mount_point(fs->private);
+	if (ret == 1)
+		logger(-1, 0, "Can't create: private dir (%s) is a mount "
+				"point. Suggestion: use --private %s/private",
+				fs->private, fs->private);
+	if (ret)
+		return VZ_FS_NEW_VE_PRVT;
+
 	/* Lock CT area */
-	if (make_dir(fs->private, 0))
+	if (make_dir(fs->private, 1))
 		return VZ_FS_NEW_VE_PRVT;
 	snprintf(tmp_dir, sizeof(tmp_dir), "%s.tmp", fs->private);
 	if (stat_file(tmp_dir) == 1) {
@@ -190,10 +236,11 @@ find:
 		ret = VZ_FS_NEW_VE_PRVT;
 		goto err;
 	}
-	dst = tmp_dir;
+	untar_to = tmp_dir;
 
 	ddata.veid = veid;
-	ddata.private = dst;
+	ddata.private = tmp_dir;
+	ddata.layout = fs->layout;
 	ch = add_cleanup_handler(cleanup_destroy_ve, &ddata);
 
 	if (ploop) {
@@ -204,7 +251,7 @@ find:
 		/* Create and mount ploop image */
 		struct vzctl_create_image_param param = {};
 		struct vzctl_mount_param mount_param = {};
-		int ploop_mode = vps_p->opt.mode;
+		int ploop_mode = fs->mode;
 
 		if (ploop_mode < 0)
 			ploop_mode = PLOOP_EXPANDED_MODE;
@@ -230,7 +277,7 @@ find:
 		/* If interrupted, umount ploop */
 		ploop_ch = add_cleanup_handler(cleanup_umount_ploop, &tmp_dir);
 
-		dst = fs->root;
+		untar_to = fs->root;
 #endif
 	}
 	if (!ploop &&
@@ -251,7 +298,7 @@ find:
 	snprintf(buf, sizeof(buf), "PRIVATE_TEMPLATE=%s", tarball);
 	i = 0;
 	env[i++] = strdup(buf);
-	snprintf(buf, sizeof(buf), "VE_PRVT=%s", dst);
+	snprintf(buf, sizeof(buf), "VE_PRVT=%s", untar_to);
 	env[i++] = strdup(buf);
 	if (!is_vz_kernel(h) && h->can_join_userns) {
 		snprintf(buf, sizeof(buf), "UID_OFFSET=%lu", uid_offset);
@@ -398,17 +445,17 @@ int vps_create(vps_handler *h, envid_t veid, vps_param *vps_p, vps_param *cmd_p,
 		goto err_cfg;
 	}
 
-	if (vps_p->opt.layout == 0) {
+	if (vps_p->res.fs.layout == 0) {
 #ifdef HAVE_PLOOP
-		vps_p->opt.layout = VE_LAYOUT_PLOOP;
+		vps_p->res.fs.layout = VE_LAYOUT_PLOOP;
 #else
-		vps_p->opt.layout = VE_LAYOUT_SIMFS;
+		vps_p->res.fs.layout = VE_LAYOUT_SIMFS;
 #endif
 	}
 
-	if (vps_p->opt.layout == VE_LAYOUT_PLOOP && !is_ploop_supported()) {
+	if (vps_p->res.fs.layout == VE_LAYOUT_PLOOP && !is_ploop_supported()) {
 		logger(-1, 0, "Alternatively, if you can't or don't want to use ploop, please");
-		logger(-1, 0, "add --layout simfs option, or set VE_LAYOUT=simfs in" GLOBAL_CFG);
+		logger(-1, 0, "add --layout simfs option, or set VE_LAYOUT=simfs in " GLOBAL_CFG);
 		ret = VZ_PLOOP_UNSUP;
 		goto err_cfg;
 	}
@@ -421,7 +468,7 @@ int vps_create(vps_handler *h, envid_t veid, vps_param *vps_p, vps_param *cmd_p,
 
 	if (action != NULL && action->mod_count) {
 		if ((ret = mod_setup(h, veid, 0, 0, action, vps_p)))
-			goto err_private;
+			goto err_cfg;
 	} else {
 		/* Set default ostemplate if not specified */
 		if (cmd_p->res.tmpl.ostmpl == NULL &&
@@ -445,7 +492,7 @@ int vps_create(vps_handler *h, envid_t veid, vps_param *vps_p, vps_param *cmd_p,
 		}
 		ret = fs_create(veid, h, vps_p);
 		if (ret)
-			goto err_root;
+			goto err_cfg;
 	}
 
 	if ((ret = vps_postcreate(veid, &vps_p->res)))
@@ -507,8 +554,7 @@ err_names:
 	remove_names(veid);
 err_root:
 	rmdir(fs->root);
-err_private:
-	vps_destroy_dir(veid, fs->private);
+	vps_destroy_dir(veid, fs->private, fs->layout);
 err_cfg:
 	if (sample_config != NULL)
 		unlink(dst);
