@@ -287,8 +287,8 @@ static int _env_create(void *data)
 	 * Technically, because clone will clone both fds, we would have to
 	 * close the other end as well. But we don't even know what it is,
 	 * since our args only include our end of the pipe. This is not a
-	 * problem because right before exec_container_init, we will call
-	 * close_fds and get away with all of them. And if we fail, we'll
+	 * problem because we will call close_fds in exec_container_init()
+	 * and get away with all of them. And if we fail, we'll
 	 * exit anywyay.
 	 */
 	if (arg->userns_p != -1)
@@ -312,10 +312,6 @@ static int _env_create(void *data)
 
 	fill_container_param(arg, &create_param);
 
-	/* Close all fds except stdin. stdin is status pipe */
-	close(STDERR_FILENO); close(STDOUT_FILENO);
-	close_fds(0, arg->wait_p, arg->err_p, -1);
-
 	return exec_container_init(arg, &create_param);
 }
 
@@ -551,6 +547,10 @@ static int ct_enter(vps_handler *h, envid_t veid, const char *root, int flags)
 		if ((!strcmp(ep->d_name, "user")))
 			continue;
 
+		/* Done later */
+		if ((!strcmp(ep->d_name, "mnt")))
+			continue;
+
 		if (snprintf(path, sizeof(path), "/proc/%d/ns/%s", task_pid, ep->d_name) < 0)
 			goto out;
 		if ((fd = open(path, O_RDONLY)) < 0)
@@ -558,9 +558,15 @@ static int ct_enter(vps_handler *h, envid_t veid, const char *root, int flags)
 		if (setns(fd, 0))
 			logger(-1, errno, "Failed to set context for %s", ep->d_name);
 		close(fd);
-
-		if (!strcmp(ep->d_name, "mnt"))
-			joined_mnt_ns = true;
+	}
+	/* Join mount ns */
+	if (snprintf(path, sizeof(path), "/proc/%d/ns/mnt", task_pid) < 0)
+		goto out;
+	if ((fd = open(path, O_RDONLY)) >= 0) {
+		if (setns(fd, 0))
+			logger(-1, errno, "Failed to set context for mnt");
+		close(fd);
+		joined_mnt_ns = true;
 	}
 
 	/*
